#ifndef _CAMERA_H_
#define _CAMERA_H_

#include "Vector3.h"
#include "Input.h"
#include "Model.h"

class CameraClass
{
private:
	enum CAMERA_TYPE
	{
		FP_CAMERA, TRACKING_CAMERA
	};

public:
	CameraClass();
	CameraClass(const CameraClass& other);
	~CameraClass();

	void Initialize(float start_x, float start_y, float start_z);
	void Initialize(Vector3* tracker, float dist_x, float dist_y, float dist_z);
	void Frame(Input* input_, float dt);
	void Frame();
	void RenderReflection(float height);
	void RenderRefraction(Vector3 objectPos);

	inline void LockMove(int x, int y, int z, float value)
	{
		if (x)
		{
			position.x = value;
			moveLockX = true;
		}
		
		if (y)
		{
			position.y = value;
			moveLockY = true;
		}

		if (z)
		{
			position.z = value;
			moveLockZ = true;
		}
	}

	inline void LockRot(int x, int y, int z, float value)
	{
		if (x)
		{
			xRotLockValue = value;
			rotLockX = true;
		}

		if (y)
		{
			yRotLockValue = value;
			rotLockY = true;
		}

		if (z)
		{
			zRotLockValue = value;
			rotation.z = value;
			rotLockZ = true;
		}
	}

	inline void SetPosition(Vector3 _position)
	{
		position = _position;
	}

	inline Vector3 GetPosition()
	{
		return position;
	}

	inline void SetRotation(Vector3 _rotation)
	{
		rotation = _rotation;
	}

	inline Vector3 GetRotation()
	{
		return rotation;
	}

	inline void SetLookAt(Vector3 _lookAt)
	{
		lookAt = _lookAt;
	};

	inline Vector3 GetLookAt()
	{
		return lookAt;
	}

	inline void SetForward(Vector3 _forward)
	{
		forward = _forward;
	}

	inline Vector3 GetForward()
	{
		return forward;
	}

	inline void SetUp(Vector3 _up)
	{
		up = _up;
	}

	inline Vector3 GetUp()
	{
		return up;
	}

	inline void SetRight(Vector3 _right)
	{
		right = _right;
	}

	inline Vector3 GetRight()
	{
		return right;
	}

	inline Vector3 GetReflectUp()
	{
		return reflectUp;
	}

	inline Vector3 GetReflectLook()
	{
		return reflectLook;
	}

	inline Vector3 GetReflectPosition()
	{
		return reflectPosition;
	}

	inline Vector3 GetRefractPosition()
	{
		return refractPosition;
	}

	inline Vector3 GetRefractLook()
	{
		return refractLook;
	}

	inline Vector3 GetRefractUp()
	{
		return refractUp;
	}

	inline Vector3* GetPositionPointer()
	{
		return &position;
	}
	
private:
	void TurnY(int mouseDiff);
	void TurnX(int mouseDiff);

	void MoveForward();
	void MoveBackward();
	void MoveLeft();
	void MoveRight();

	void CalculateAngles();

	void Update();

	void EnforceLocking();

private:
	Vector3 position;
	Vector3 rotation;
	Vector3 lookAt;
	Vector3 forward;
	Vector3 up;
	Vector3 right;

	Vector3 reflectUp;
	Vector3 reflectPosition;
	Vector3 reflectLook;

	Vector3 refractUp;
	Vector3 refractPosition;
	Vector3 refractLook;

	float forwardMoveSpeed;
	float backwardMoveSpeed;
	float leftMoveSpeed;
	float rightMoveSpeed;

	float prevMouseX = 0.0f, prevMouseY = 0.0f;

	CAMERA_TYPE m_cameraType;
	Vector3 m_trackDistance;
	Vector3* m_trackingTarget;

	bool moveLockX, moveLockY, moveLockZ;
	float xMoveLockValue, yMoveLockValue, zMoveLockValue;
	bool rotLockX, rotLockY, rotLockZ;
	float xRotLockValue, yRotLockValue, zRotLockValue;

	float dt;
};

#endif