#include "FontModel.h"

bool FontModel::Initialize(Font* font, char* sentence, float posX, float posY, float r, float g, float b)
{
	m_font = font;
	m_sentence = sentence;

	m_textR = r;
	m_textG = g;
	m_textB = b;

	SetTranslation(posX, posY, 0.0f);

	BuildVertexArray();

	return true;
}

void FontModel::UpdateFont(Font* font)
{
	m_font = font;

	BuildVertexArray();
}

void FontModel::UpdateSentence(char* sentence)
{
	m_sentence = sentence;

	BuildVertexArray();
}

void FontModel::SetTextRGB(float r, float g, float b)
{
	m_textR = r;
	m_textG = g;
	m_textB = b;
}

void FontModel::BuildVertexArray()
{
	int numLetters, index;
	float drawX, drawY, letterLeft, letterRight, letterSize;

	drawX = 0.0f;
	drawY = 0.0f;

	numLetters = (int)strlen(m_sentence);

	m_vertexCount = numLetters * 6;

	m_vertices = new VertexType[m_vertexCount];
	
	index = 0;

	for (int i = 0; i < numLetters; i++)
	{
		if (m_sentence[i] != ' ')
		{
			m_font->GetCharCoordinates(m_sentence[i], letterLeft, letterRight, letterSize);
			
			//First triangle
			m_vertices[index].x = drawX;
			m_vertices[index].y = drawY;
			m_vertices[index].z = 0.0f;
			m_vertices[index].tu = letterLeft;
			m_vertices[index].tv = 0.0f;
			index++;

			m_vertices[index].x = drawX + letterSize;
			m_vertices[index].y = drawY;
			m_vertices[index].z = 0.0f;
			m_vertices[index].tu = letterRight;
			m_vertices[index].tv = 0.0f;
			index++;

			m_vertices[index].x = drawX + letterSize;
			m_vertices[index].y = drawY - 16;
			m_vertices[index].z = 0.0f;
			m_vertices[index].tu = letterRight;
			m_vertices[index].tv = 1.0f;
			index++;

			//Second triangle
			m_vertices[index].x = drawX;
			m_vertices[index].y = drawY;
			m_vertices[index].z = 0.0f;
			m_vertices[index].tu = letterLeft;
			m_vertices[index].tv = 0.0f;
			index++;

			m_vertices[index].x = drawX + letterSize;
			m_vertices[index].y = drawY - 16;
			m_vertices[index].z = 0.0f;
			m_vertices[index].tu = letterRight;
			m_vertices[index].tv = 1.0f;
			index++;

			m_vertices[index].x = drawX;
			m_vertices[index].y = drawY - 16;
			m_vertices[index].z = 0.0f;
			m_vertices[index].tu = letterLeft;
			m_vertices[index].tv = 1.0f;
			index++;

			drawX = drawX + letterSize + 1.0f;
		}
		else
		{
			drawX = drawX + 3.0f;
		}


	}
}

void FontModel::RenderHierarchy()
{
	glBindTexture(GL_TEXTURE_2D, m_font->GetFontTexture());
	glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_WRAP_S, GL_CLAMP);
	glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_WRAP_T, GL_CLAMP);

	float x, y, z;

	glPushMatrix();
	GetTranslation(x, y, z);
	glTranslatef(x, y, z);
	GetScaling(x, y, z);
	glScalef(x, y, z);
	GetRotation(x, y, z);
	glRotatef(z, 0, 0, 1);
	glRotatef(y, 0, 1, 0);
	glRotatef(x, 1, 0, 0);

	glDisable(GL_LIGHTING);
	glDisable(GL_DEPTH);
	glDisable(GL_FOG);

	glEnable(GL_BLEND);
	glBlendFunc(GL_SRC_ALPHA, GL_ONE);

	glBegin(GL_TRIANGLES);

	for (int i = 0; i < m_vertexCount; i++)
	{
		glTexCoord2f(m_vertices[i].tu, m_vertices[i].tv);
		//glColor3f(m_textR, m_textG, m_textB);
		glVertex3f(m_vertices[i].x, m_vertices[i].y, m_vertices[i].z);
	}


	glEnd();


	glEnable(GL_LIGHTING);
	glEnable(GL_DEPTH);
	glEnable(GL_FOG);

	glDisable(GL_BLEND);

}

void FontModel::Render()
{

	glBindTexture(GL_TEXTURE_2D, m_font->GetFontTexture());
	glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_WRAP_S, GL_CLAMP);
	glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_WRAP_T, GL_CLAMP);

	float x, y, z;

	
	glPushMatrix();
	GetTranslation(x, y, z);
	glTranslatef(x, y, z);
	GetScaling(x, y, z);
	glScalef(x, y, z);
	GetRotation(x, y, z);
	glRotatef(z, 0, 0, 1);
	glRotatef(y, 0, 1, 0);
	glRotatef(x, 1, 0, 0);

	
	glDisable(GL_LIGHTING);
	glDisable(GL_DEPTH);
	
	glEnable(GL_BLEND);
	glBlendFunc(GL_SRC_ALPHA, GL_ONE);
	
	glBegin(GL_TRIANGLES);

	for (int i = 0; i < m_vertexCount; i++)
	{
		glTexCoord2f(m_vertices[i].tu, m_vertices[i].tv);
		//glColor3f(m_textR, m_textG, m_textB);
		glVertex3f(m_vertices[i].x, m_vertices[i].y, m_vertices[i].z);
	}

	
	glEnd();
	glPopMatrix();
	
	glEnable(GL_LIGHTING);
	glEnable(GL_DEPTH);

	glDisable(GL_BLEND);
	
}