#ifndef _MATERIAL_H_
#define _MATERIAL_H_

#include <gl/glew.h>

#include "Vector3.h"

class Material
{
public:
	Material()
	{
		m_ambientColor = Vector3(0.0f, 0.0f, 0.0f);
		m_diffuseColor = Vector3(0.0f, 0.0f, 0.0f);
		m_specularColor = Vector3(0.0f, 0.0f, 0.0f);
		m_specularExponent = 0.0f;
		m_transparency = 0.0f;
		m_illuminationMode = 0;
	}

	inline void SetAmbientColor(float r, float g, float b)
	{
		m_ambientColor = Vector3(r, g, b);
	}

	inline Vector3 GetAmbientColor()
	{
		return m_ambientColor;
	}

	inline void SetDiffuseColor(float r, float g, float b)
	{
		m_diffuseColor = Vector3(r, g, b);
	}

	inline Vector3 GetDiffuseColor()
	{
		return m_diffuseColor;
	}

	inline void SetSpecularColor(float r, float g, float b)
	{
		m_specularColor = Vector3(r, g, b);
	}

	inline Vector3 GetSpecularColor()
	{
		return m_specularColor;
	}

	inline void SetSpecularExponent(float specular_exponent)
	{
		m_specularColor = specular_exponent;
	}

	inline float GetSpecularExponent()
	{
		return m_specularExponent;
	}

	inline void SetTransparency(float transparency)
	{
		m_transparency = transparency;
	}

	inline float GetTransparency()
	{
		return m_transparency;
	}

	inline void SetIlluminationMode(int illum_mode)
	{
		m_illuminationMode = illum_mode;
	}

	inline int GetIlluminationMode()
	{
		return m_illuminationMode;
	}

	inline GLuint GetDiffuseTexture()
	{
		return m_diffuseTexture;
	}

	inline GLuint GetAlphaTexture()
	{
		return m_alphaTexture;
	}

	inline GLuint GetAmbientTexture()
	{
		return m_ambientTexture;
	}

	inline GLuint GetSpecularTexture()
	{
		return m_specularTexture;
	}

	inline GLuint GetSpecularHighlight()
	{
		return m_specularHighlight;
	}

	inline GLuint GetBumpMap()
	{
		return m_bumpMap;
	}

private:
	Vector3 m_ambientColor;
	Vector3 m_diffuseColor;
	Vector3 m_specularColor;
	float m_specularExponent;
	float m_transparency;
	int m_illuminationMode;
	GLuint m_diffuseTexture;
	GLuint m_ambientTexture;
	GLuint m_alphaTexture;
	GLuint m_specularTexture;
	GLuint m_specularHighlight;
	GLuint m_bumpMap;

};

#endif