#ifndef _MODEL_H_
#define _MODEL_H_

#define M_PI 3.14159265358979323846
#include <fstream>
#include <math.h>
#include <chrono>
#include <list>
#include <Windows.h>
#include <stdio.h>
#include <gl/glew.h>
#include <gl/GLU.h>
#include <gl/glm.hpp>
#include <SOIL.h>
#include <algorithm>
#include <thread>

#include "Vector3.h"
#include "BumpMap.h"

using namespace std;

struct VertexType
{
	float x, y, z; //Local Position
	float tu, tv; //Texture Coords
	float nx, ny, nz; //Vertex Normals
	float lerpX, lerpY, lerpZ; //Linear Interpolation Target
	float iniX, iniY, iniZ; //Linear Interpolation Start
	float iniNx, iniNy, iniNz;
	float lerpS;
	bool lerpD;
	VertexType()
	{
		x = 0.0f;
		y = 0.0f;
		z = 0.0f;
	}
};

struct TempVertex
{
	float x1, y1, z1;
	float x2, y2, z2;
	float x3, y3, z3;
};

struct LoadVertex
{
	float x, y, z;
};

struct LoadFace
{
	int vIndex1, vIndex2, vIndex3;
	int tIndex1, tIndex2, tIndex3;
	int nIndex1, nIndex2, nIndex3;
};

struct ShadowThreadArgs
{
	int numVertices;
	void* terrainPointer;

};

class Model
{
public:
	Model();
	Model(const Model&);
	~Model();

	virtual void Render();
	virtual void Shutdown();

	int GetVertexCount();
	VertexType GetVertex(int vertexNum);
	
	int GetIndexCount();
	long GetIndex(int indexNum);

	float GetTransformMatrixValue(int, int);

	void GetTranslation(float&, float&, float&);
	void GetRotation(float&, float&, float&);
	void GetScaling(float&, float&, float&);

	void SetTranslation(float, float, float);
	void SetRotation(float, float, float);
	void SetScaling(float, float, float);
	void CalculateTransformMatrix();

	void LoadTexture(char* filename);
	void SetTexture(GLuint texture);
	GLuint GetTexture();

	void LoadBumpMap(char* filename);
	Vector3 SampleBumpMap(float tu, float tv);

	void GenerateShadowMatrix(float matrix[16], float light_pos[4], GLfloat v0[3], GLfloat v1[3], GLfloat v2[3]);
	void RenderPlanarShadow(float light_pos[4]);
	void RenderShadow();

	void CreateShadowImposter(void* terrainPointer);
	void SetShadowImposter(GLuint imposterTexture);

protected:
	void MatrixMultiply(float[4][4], float[4][4], float[4][4]);
	void CalculateShadowVertexPosition(ShadowThreadArgs args);

public:
	bool isIndexed;

protected:
	VertexType* m_vertices;
	VertexType* m_shadowVertices;
	unsigned long* m_indices;
	int m_vertexCount;
	int m_indexCount;
	GLuint m_texture;
	GLuint m_imposterTexture;
	BumpMap* m_bumpMap;
	float TransformationMatrix[4][4];
	float m_shadowMatrix[16];
	Vector3 m_translate;
	Vector3 m_rotate;
	Vector3 m_scale;

};

#endif