#include "Scene3D.h"

bool Scene3D::CreatePixelFormat(HDC hdc) 
{ 
    PIXELFORMATDESCRIPTOR pfd = {0}; 
    int pixelformat; 
 
    pfd.nSize = sizeof(PIXELFORMATDESCRIPTOR);	// Set the size of the structure
    pfd.nVersion = 1;							// Always set this to 1
	// Pass in the appropriate OpenGL flags
    pfd.dwFlags = PFD_DRAW_TO_WINDOW | PFD_SUPPORT_OPENGL | PFD_DOUBLEBUFFER; 
    pfd.dwLayerMask = PFD_MAIN_PLANE;			// standard mask (this is ignored anyway)
    pfd.iPixelType = PFD_TYPE_RGBA;				// RGB and Alpha pixel type
    pfd.cColorBits = COLOUR_DEPTH;				// Here we use our #define for the color bits
    pfd.cDepthBits = COLOUR_DEPTH;				// Ignored for RBA
    pfd.cAccumBits = 0;							// nothing for accumulation
    pfd.cStencilBits = 0;						// nothing for stencil
 
	//Gets a best match on the pixel format as passed in from device
    if ( (pixelformat = ChoosePixelFormat(hdc, &pfd)) == false ) 
    { 
        MessageBox(NULL, "ChoosePixelFormat failed", "Error", MB_OK); 
        return false; 
    } 
 
	//sets the pixel format if its ok. 
    if (SetPixelFormat(hdc, pixelformat, &pfd) == false) 
    { 
        MessageBox(NULL, "SetPixelFormat failed", "Error", MB_OK); 
        return false; 
    } 
 
    return true;
}

void Scene3D::ResizeGLWindow(int width, int height)// Initialize The GL Window
{
	if (height==0)// Prevent A Divide By Zero error
	{
		height=1;// Make the Height Equal One
	}

	s_width = width;
	s_height = height;

	glViewport(0,0,width,height);

	glMatrixMode(GL_PROJECTION);
	glLoadIdentity();

	//calculate aspect ratio
	gluPerspective(45.0f,(GLfloat)width/(GLfloat)height, 0.01f , 300.0f);

	glMatrixMode(GL_MODELVIEW);// Select The Modelview Matrix
	glLoadIdentity();// Reset The Modelview Matrix
}

void Scene3D::UpdateProjection(bool toggle)
{
	static bool usePerspective = true;

	if (toggle)
		usePerspective = !usePerspective;

	glMatrixMode(GL_PROJECTION);
	glLoadIdentity();

	if (usePerspective)
	{
		gluPerspective(45.0f, (GLfloat)s_width / (GLfloat)s_height, 0.01f, 300.0f);
	}
	else
	{
		gluOrtho2D(-(s_width/2), (s_width/2), -(s_height/2), (s_height/2));
	}

	glMatrixMode(GL_MODELVIEW);
	glLoadIdentity();

	glViewport(0, 0, s_width, s_height);
}

void Scene3D::InitializeOpenGL(int width, int height) 
{  
    hdc = GetDC(*hwnd);//  sets  global HDC

    if (!CreatePixelFormat(hdc))//  sets  pixel format
        PostQuitMessage (0);

    hrc = wglCreateContext(hdc);	//  creates  rendering context from  hdc
    wglMakeCurrent(hdc, hrc);		//	Use this HRC

	ResizeGLWindow(width, height);	// Setup the Screen
}

void Scene3D::Init(HWND* wnd, Input* in)
{
	DirectionalLightData directionalData;
	
	hwnd = wnd;
	input = in;

	light_names[0] = GL_LIGHT0;
	light_names[1] = GL_LIGHT1;
	light_names[2] = GL_LIGHT2;
	light_names[3] = GL_LIGHT3;
	light_names[4] = GL_LIGHT4;
	light_names[5] = GL_LIGHT5;
	light_names[6] = GL_LIGHT6;
	light_names[7] = GL_LIGHT7;

	ambientLight.r = 0.1f;
	ambientLight.g = 0.1f;
	ambientLight.b = 0.1f;
	ambientLight.a = 1.0f;

	directionalData.r = 0.5f;
	directionalData.g = 0.5f;
	directionalData.b = 0.5f;
	directionalData.a = 1.0f;

	directionalData.dirX = 1.0f;
	directionalData.dirY = 1.0f;
	directionalData.dirZ = 0.0f;

	directionalLights.push_back(directionalData);

	intensity = 1.0f;
	radius = 10.0f;
	attenuation_amount = 1.0f;
	attenuation_type = AttenuationType::QUADRATIC;

	GetClientRect(*hwnd, &screenRect);	//get rect into our handy global rect
	InitializeOpenGL(screenRect.right, screenRect.bottom); // initialise openGL

	//OpenGL settings
	glShadeModel(GL_SMOOTH);							// Enable Smooth Shading
	//glClearColor(0.39f, 0.58f, 93.0f, 1.0f);				// Black Background
	glClearColor(0.0f, 0.0f, 0.0f, 1.0f);
	glClearDepth(1.0f);									// Depth Buffer Setup
	glEnable(GL_DEPTH_TEST);							// Enables Depth Testing
	glDepthFunc(GL_LEQUAL);								// The Type Of Depth Testing To Do
	glHint(GL_PERSPECTIVE_CORRECTION_HINT, GL_FASTEST);	// Use faster perspective calculations to save time
	glEnable(GL_LIGHTING);
	//glEnable(GL_COLOR_MATERIAL);
	glewInit();

	m_camera.Initialize(1.0f, 3.0f, 0.0f);

	m_Font = new Font;

	m_Font->Initialize("data/font.png", "data/fontdata.txt");

	m_heightText = new FontModel;

	m_heightText->Initialize(m_Font, "Hello World!", -(s_width / 2) + 10.0f, (s_height / 2) - 230.0f, 1.0f, 1.0f, 1.0f);

	m_typeText = new FontModel;

	m_typeText->Initialize(m_Font, "Attenuation Type: QUADRATIC", -(s_width / 2) + 10.0f, (s_height / 2) - 210.0f, 1.0f, 1.0f, 1.0f);

	m_intensityText = new FontModel;

	m_intensityText->Initialize(m_Font, "Hello World!", -(s_width / 2) + 10.0f, (s_height / 2) - 190.0f, 1.0f, 1.0f, 1.0f);

	m_attenuationText = new FontModel;

	m_attenuationText->Initialize(m_Font, "Hello World!", -(s_width / 2) + 10.0f, (s_height / 2) - 250.0f, 1.0f, 1.0f, 1.0f);

	m_radiusText = new FontModel;

	m_radiusText->Initialize(m_Font, "Hello World!", -(s_width / 2) + 10.0f, (s_height / 2) - 270.0f, 1.0f, 1.0f, 1.0f);


	m_Skybox = new SkyboxModel;

	m_Skybox->Initialize();
	
	m_terrain = new TerrainModel;

	the_clock::time_point begin = the_clock::now();

	m_terrain->Initialize("data/flatland.r16");

	the_clock::time_point finish = the_clock::now();

	auto full_time = duration_cast<milliseconds>(finish - begin).count();
	
	std::ostringstream os_;
	os_ << "Building the terrain took " << full_time << " ms." << endl;
	OutputDebugString(os_.str().c_str());

	m_terrain->LoadTexture("data/rock.png");

	float light_pos[4] = { -4.0f, 5.0f, 0.0f, 1.0f };

	float objX, objZ, objHeight;

	//Enable texturing and set the texturing parameters
	glEnable(GL_TEXTURE_2D);
	glTexEnvf(GL_TEXTURE_ENV, GL_TEXTURE_ENV_MODE, GL_MODULATE);
	glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_WRAP_S, GL_REPEAT);
	glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_WRAP_T, GL_REPEAT);

}

bool Scene3D::Update(float dt)
{
	bool foundHeight;
	float height, fps;
	char tempString[24];
	char fpsString[24];
	Vector3 prevPosition;

	fps = 1.0f / dt;

	if (fps > 9999)
	{
		fps = 9999;
	}

	// Do important update here
	// Handle user input, update variables etc
	

	if (input->isKeyDown('T'))
	{
		input->SetKeyUp('T');
		followTerrain = !followTerrain;
	}

	if (input->isKeyDown(VK_TAB))
	{
		input->SetKeyUp(VK_TAB);
		switch (attenuation_type)
		{
		case AttenuationType::CONSTANT:
			attenuation_type = AttenuationType::LINEAR;
			m_typeText->UpdateSentence("Attenuation Type: LINEAR");
			break;
		case AttenuationType::LINEAR:
			attenuation_type = AttenuationType::QUADRATIC;
			m_typeText->UpdateSentence("Attenuation Type: QUADRATIC");
			break;
		case AttenuationType::QUADRATIC:
			attenuation_type = AttenuationType::CONSTANT;
			m_typeText->UpdateSentence("Attenuation Type: CONSTANT");
			break;
		}
	}

	if (input->isKeyDown(VK_CONTROL))
	{
		input->SetKeyUp(VK_CONTROL);
		intensity--;
	}

	if (input->isKeyDown(VK_SHIFT))
	{
		input->SetKeyUp(VK_SHIFT);
		intensity++;
	}

	if (input->isKeyDown(VK_RETURN))
	{
		input->SetKeyUp(VK_RETURN);
		m_terrain->WriteSnapshot("test.txt");
	}

	if (input->isKeyDown(VK_SPACE))
	{
		input->SetKeyUp(VK_SPACE);
		m_terrain->ManipulateTerrain(m_camera.GetPosition(), radius, intensity, attenuation_amount, attenuation_type);
	}

	if (input->isKeyDown('Q'))
	{
		input->SetKeyUp('Q');
		radius++;
	}

	if (input->isKeyDown('E'))
	{
		input->SetKeyUp('E');
		radius--;
	}

	if (input->isKeyDown('R'))
	{
		input->SetKeyUp('R');
		attenuation_amount+= 0.1f;
	}

	if (input->isKeyDown('F'))
	{
		input->SetKeyUp('F');
		attenuation_amount-= 0.1f;
	}

	if (input->isKeyDown(VK_ESCAPE))
	{
		return true;
	}

	m_terrain->Frame();

	prevPosition = m_camera.GetPosition();

	m_camera.Frame(input, dt);

	m_Skybox->Frame(dt);

	Vector3 position = m_camera.GetPosition();
	Vector3 normal;
	if (followTerrain)
	{
		foundHeight = static_cast<TerrainModel*>(m_terrain)->GetHeightAtPosition(position.x, position.z, height);
		if (foundHeight)
		{
			m_camera.SetPosition(Vector3(position.x, height + 2.0f, position.z));
		}
		
		foundHeight = reinterpret_cast<TerrainModel*>(m_terrain)->GetNormalAtPosition(position.x, position.z, normal);
		if (foundHeight && normal.y < 0.65)
		{
			m_camera.SetPosition(prevPosition);
		}
	}

	position = m_camera.GetPosition();

	_itoa_s(position.y, tempString, 10);

	strcpy_s(fpsString, "Height: ");
	strcat_s(fpsString, tempString);

	m_heightText->UpdateSentence(fpsString);
	
	_itoa_s(intensity, tempString, 10);

	strcpy_s(fpsString, "Intensity: ");
	strcat_s(fpsString, tempString);

	m_intensityText->UpdateSentence(fpsString);

	_itoa_s(radius, tempString, 10);

	strcpy_s(fpsString, "Radius: ");
	strcat_s(fpsString, tempString);

	m_radiusText->UpdateSentence(fpsString);

	_itoa_s(attenuation_amount * 10, tempString, 10);

	strcpy_s(fpsString, "Attenuation: ");
	strcat_s(fpsString, tempString);

	m_attenuationText->UpdateSentence(fpsString);


	Render(&m_camera, 0, false, true, 0, 0);
	
	return false;
}

void Scene3D::Render(CameraClass* camera, GLuint drawBuffer, bool wireframe, bool renderUI, int width, int height)
{
	glBindFramebuffer(GL_FRAMEBUFFER, drawBuffer);
	
	glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);// Clear The Screen And The Depth Buffer
	
	glLoadIdentity();// load Identity Matrix

	Vector3 position = camera->GetPosition();
	Vector3 lookAt = camera->GetLookAt();
	Vector3 up = camera->GetUp();
	
	gluLookAt(position.x, position.y, position.z, lookAt.x, lookAt.y, lookAt.z, up.x, up.y, up.z);

	m_Skybox->SetTranslation(position.x, position.y, position.z);

	m_Skybox->CalculateTransformMatrix();

	m_Skybox->Render();

	SetLights();

	m_terrain->Render();

	for (int i = 0; i < 8; i++)
	{
		glDisable(light_names[i]);
	}

	UpdateProjection(true);

	m_heightText->Render();

	m_radiusText->Render();

	m_attenuationText->Render();

	m_typeText->Render();

	m_intensityText->Render();
	
	UpdateProjection(true);

	if (drawBuffer == 0)
		SwapBuffers(hdc);// Swap the frame buffers.
}

void Scene3D::BlendTextures(GLuint texA, GLuint texB, float blendFactor)
{
	glActiveTexture(GL_TEXTURE0);
	glBindTexture(GL_TEXTURE_2D, texA);
	glEnable(GL_TEXTURE_2D);
	glTexEnvf(GL_TEXTURE_ENV, GL_TEXTURE_ENV_MODE, GL_REPLACE);

	glActiveTexture(GL_TEXTURE1);
	glBindTexture(GL_TEXTURE_2D, texB);
	glEnable(GL_TEXTURE_2D);
	glTexEnvf(GL_TEXTURE_ENV, GL_TEXTURE_ENV_MODE, GL_COMBINE);
	
	glTexEnvf(GL_TEXTURE_ENV, GL_COMBINE_RGB, GL_INTERPOLATE);

	glTexEnvf(GL_TEXTURE_ENV, GL_SOURCE0_RGB, GL_PREVIOUS);

	glTexEnvf(GL_TEXTURE_ENV, GL_SOURCE1_RGB, GL_TEXTURE);
	
	glTexEnvf(GL_TEXTURE_ENV, GL_SOURCE2_RGB, GL_CONSTANT);

	glTexEnvf(GL_TEXTURE_ENV, GL_OPERAND2_RGB, GL_SRC_ALPHA);

	//Set the blend factor
	float myColor[4] = { 0.0f, 0.0f, 0.0f, blendFactor };

	glTexEnvfv(GL_TEXTURE_ENV, GL_TEXTURE_ENV_COLOR, myColor);

}

void Scene3D::DrawIndexedModel(Model* model)
{
	Vector3 normalVector;

	for (int i = 0; i < model->GetIndexCount(); ++i)
	{
		VertexType vertex = model->GetVertex(model->GetIndex(i));

		normalVector = model->SampleBumpMap(vertex.tu, vertex.tv);
		glNormal3f(vertex.nx, vertex.ny, vertex.nz);
		glMultiTexCoord2f(GL_TEXTURE0, vertex.tu + (normalVector.x *0.01f), vertex.tv + (normalVector.y * 0.01f));
		glMultiTexCoord2f(GL_TEXTURE1, vertex.tu + (normalVector.x * 0.01f), vertex.tv + (normalVector.y * 0.01f));
		glVertex3f(vertex.x, vertex.y, vertex.z);
	}


}

void Scene3D::SetLights()
{
	int light_num = 0;

	for (int i = 0; i < directionalLights.size(); i++)
	{
		if (light_num < 8)
		{
			if (light_num == 0)
			{
				GLfloat Light_Ambient[] = { ambientLight.r, ambientLight.g, ambientLight.b, 1.0f };
				glLightfv(light_names[light_num], GL_AMBIENT, Light_Ambient);
			}

			GLfloat Light_Diffuse[] = { directionalLights[i].r, directionalLights[i].g, directionalLights[i].b, directionalLights[i].a };
			GLfloat Light_Position[] = { directionalLights[i].dirX, directionalLights[i].dirY, directionalLights[i].dirZ, 0.0f };

			glLightfv(light_names[light_num], GL_DIFFUSE, Light_Diffuse);
			glLightfv(light_names[light_num], GL_POSITION, Light_Position);
			glEnable(light_names[light_num]);
			light_num++;
		}
	}

	for (int i = 0; i < pointLights.size(); i++)
	{
		if (light_num < 8)
		{
			if (light_num == 0)
			{
				GLfloat Light_Ambient[] = { ambientLight.r, ambientLight.g, ambientLight.b, 1.0f };
				glLightfv(light_names[light_num], GL_AMBIENT, Light_Ambient);
			}

			GLfloat Light_Diffuse[] = { pointLights[i].r, pointLights[i].g, pointLights[i].b, pointLights[i].a };
			GLfloat Light_Position[] = { pointLights[i].posX, pointLights[i].posY, pointLights[i].posZ, 1.0f };

			glLightfv(light_names[light_num], GL_DIFFUSE, Light_Diffuse);
			glLightfv(light_names[light_num], GL_POSITION, Light_Position);
			glEnable(light_names[light_num]);
			light_num++;
		}
	}

	for (int i = 0; i < spotLights.size(); i++)
	{
		if (light_num < 8)
		{
			if (light_num == 0)
			{
				GLfloat Light_Ambient[] = { ambientLight.r, ambientLight.g, ambientLight.b, 1.0f };
				glLightfv(light_names[light_num], GL_AMBIENT, Light_Ambient);
			}

			GLfloat Light_Diffuse[] = { spotLights[i].r, spotLights[i].g, spotLights[i].b, spotLights[i].a };
			GLfloat Light_Position[] = { spotLights[i].posX, spotLights[i].posY, spotLights[i].posZ, 1.0f };
			GLfloat Light_Direction[] = { spotLights[i].dirX, spotLights[i].dirY, spotLights[i].dirZ };

			glLightfv(light_names[light_num], GL_DIFFUSE, Light_Diffuse);
			glLightfv(light_names[light_num], GL_POSITION, Light_Position);
			glLightf(light_names[light_num], GL_SPOT_CUTOFF, spotLights[i].spot_cutoff);
			glLightfv(light_names[light_num], GL_SPOT_DIRECTION, Light_Direction);
			glLightf(light_names[light_num], GL_SPOT_EXPONENT, spotLights[i].spot_exponent);
			glEnable(light_names[light_num]);
			light_num++;
		}
	}
}

void Scene3D::Shutdown()
{
	m_terrain->Shutdown();
	delete m_terrain;
	m_terrain = nullptr;
}

void Scene3D::Resize()
{
	if(hwnd == NULL)
		return;

	GetClientRect(*hwnd, &screenRect);	
	ResizeGLWindow(screenRect.right, screenRect.bottom);	

	//Move text elements to their correct screen positions
	m_heightText->SetTranslation(-(s_width / 2) + 10.0f, (s_height / 2) - 230.0f, 0.0f);
	m_typeText->SetTranslation(-(s_width / 2) + 10.0f, (s_height / 2) - 210.0f, 0.0f);
	m_intensityText->SetTranslation(-(s_width / 2) + 10.0f, (s_height / 2) - 190.0f, 0.0f);
	m_attenuationText->SetTranslation(-(s_width / 2) + 10.0f, (s_height / 2) - 250.f, 0.0f);
	m_radiusText->SetTranslation(-(s_width / 2) + 10.0f, (s_height / 2) - 270.0f, 0.0f);
}