#ifndef SCENE3D_H
#define SCENE3D_H

#define _USE_MATH_DEFINES

#include <windows.h>
#include <stdio.h>
#include <mmsystem.h>
#include <math.h>
#include <gl/glew.h>
#include <gl/gl.h>
#include <gl/glu.h>
#include <vector>
#include <SOIL.h>
#include <thread>
#include <chrono>
#include <iostream>
#include <sstream>

#include "Input.h"
#include "TerrainModel.h"
#include "SkyboxModel.h"
#include "Font.h"
#include "FontModel.h"
#include "Vector3.h"
#include "Camera.h"

using std::thread;
using std::chrono::duration_cast;
using std::chrono::milliseconds;
typedef std::chrono::steady_clock the_clock;


#define COLOUR_DEPTH 16	//Colour depth

class Scene3D
{
private:
	struct LightData
	{
		float r, g, b, a;
	};

	struct AmbientLightData : LightData
	{

	};

	struct PointLightData : LightData
	{
		float posX, posY, posZ;
		float const_attenuation;
		float linear_attenuation;
		float quadratic_attenuation;
	};

	struct DirectionalLightData : LightData
	{
		float dirX, dirY, dirZ;
	};

	struct SpotLightData : LightData
	{
		float dirX, dirY, dirZ;
		float posX, posY, posZ;
		float spot_cutoff;
		float spot_exponent;
		float const_attenuation;
		float linear_attenuation;
		float quadratic_attenuation;
	};

	
public:
	void Init(HWND* hwnd, Input* in);	//initialse function
	bool Update(float dt);
	void Render(CameraClass* camera, GLuint drawBuffer, bool wireframe, bool renderUI, int width, int height);	// render scene
	void Resize();
	void Shutdown();

protected:
	bool CreatePixelFormat(HDC hdc);
	void ResizeGLWindow(int width, int height);	//width and height
	void InitializeOpenGL(int width, int height); // width and height
	void SetLights();
	void DrawIndexedModel(Model* model);
	void BlendTextures(GLuint texA, GLuint texB, float blendFactor);
	void UpdateProjection(bool toggle);

	//vars
	HWND* hwnd;
	Input* input;
	RECT screenRect;
	HDC	hdc;
	HGLRC hrc;			//hardware RENDERING CONTEXT
	int s_width, s_height;
	SkyboxModel* m_Skybox;
	Font* m_Font;
	FontModel* m_heightText;
	FontModel* m_typeText;
	FontModel* m_intensityText;
	FontModel* m_radiusText;
	FontModel* m_attenuationText;
	TerrainModel* m_terrain;
	Model* m_water;
	AmbientLightData ambientLight;
	std::vector<DirectionalLightData> directionalLights;
	std::vector<PointLightData> pointLights;
	std::vector<SpotLightData> spotLights;
	CameraClass m_camera;
	GLenum light_names[8];
	bool followTerrain;
	float intensity;
	float radius;
	float attenuation_amount;
	AttenuationType attenuation_type;
};

#endif