#include "SkyboxModel.h"

bool SkyboxModel::Initialize()
{
	bool result;

	result = LoadSkyDome("data/skydome.txt");
	if (!result)
	{
		return false;
	}

	m_apexColor[0] = 0.05f;
	m_apexColor[1] = 0.0f;
	m_apexColor[2] = 0.1f;
	m_apexColor[3] = 1.0f;
	
	m_centerColor[0] = 0.5f;
	m_centerColor[1] = 0.0f;
	m_centerColor[2] = 0.15f;
	m_centerColor[3] = 1.0f;
	
	m_cloudPlane = new SkyPlaneModel;
	if (!m_cloudPlane)
	{
		return false;
	}

	result = m_cloudPlane->Initialize("data/cloud001.png", "data/cloud002.png", "data/perturb001.bmp");
	if (!result)
	{
		return false;
	}

	return true;
}

bool SkyboxModel::LoadSkyDome(char* filename)
{
	ifstream fin;
	char input;
	int i;

	//Open the model file
	fin.open(filename);

	//If it could not open the file then exit
	if (fin.fail())
	{
		return false;
	}

	//Read up to the value of vertex count
	fin.get(input);
	while (input != ':')
	{
		fin.get(input);
	}

	//Read in the vertex count
	fin >> m_vertexCount;

	//Set the number of indices to be the same as the vertex count
	m_indexCount = m_vertexCount;

	//Create the model using the vertex count that was read in
	m_vertices = new VertexType[m_vertexCount];
	if (!m_vertices)
	{
		return false;
	}

	//Read up to the beginning of the data
	fin.get(input);
	while (input != ':')
	{
		fin.get(input);
	}
	fin.get(input);
	fin.get(input);

	//Read in the vertex data
	for (i = 0; i < m_vertexCount; i++)
	{
		fin >> m_vertices[i].x >> m_vertices[i].y >> m_vertices[i].z;
		fin >> m_vertices[i].tu >> m_vertices[i].tv;
		fin >> m_vertices[i].nx >> m_vertices[i].ny >> m_vertices[i].nz;
	}

	//Close the model file
	fin.close();

	return true;
}

void SkyboxModel::Shutdown()
{
	if (m_cloudPlane)
	{
		m_cloudPlane->Shutdown();
		delete m_cloudPlane;
		m_cloudPlane = nullptr;
	}

	if (m_vertices)
	{
		delete[] m_vertices;
		m_vertices = nullptr;
	}
}

void SkyboxModel::Frame(float dt)
{
	m_cloudPlane->Frame(dt);

}

void SkyboxModel::Render()
{
	float height;
	float skyColor[4];

	glBindTexture(GL_TEXTURE_2D, 0);

	glDisable(GL_DEPTH_TEST);
	glDisable(GL_LIGHTING);

	float x, y, z;

	glPushMatrix();
	
	GetTranslation(x, y, z);
	glTranslatef(x, y, z);
	GetScaling(x, y, z);
	glScalef(x, y, z);

	glBegin(GL_TRIANGLES);

	for (int i = 0; i < m_vertexCount; ++i)
	{
		height = m_vertices[i].y;

		if (height < 0.0f)
		{
			height = 0.0f;
		}

		skyColor[0] = lerp(m_centerColor[0], m_apexColor[0], height);
		skyColor[1] = lerp(m_centerColor[1], m_apexColor[1], height);
		skyColor[2] = lerp(m_centerColor[2], m_apexColor[2], height);
		
		glColor4f(skyColor[0], skyColor[1], skyColor[2], 1.0f);
		glVertex3f(m_vertices[i].x, m_vertices[i].y, m_vertices[i].z);
	}

	glEnd();

	m_cloudPlane->CalculateTransformMatrix();

	m_cloudPlane->Render();

	glPopMatrix();

	glEnable(GL_DEPTH_TEST);
	glEnable(GL_LIGHTING);
}