#include "SkyplaneModel.h"

bool SkyPlaneModel::Initialize(char* cloudTextureA, char* cloudTextureB, char* perturbTexture)
{
	int skyPlaneResolution, textureRepeat;
	float skyPlaneWidth, skyPlaneTop, skyPlaneBottom;
	bool result;

	skyPlaneResolution = 20;
	skyPlaneWidth = 10.0f;
	skyPlaneTop = 0.5f;
	skyPlaneBottom = -0.2f;
	textureRepeat = 4;

	m_brightness = 0.75f;

	m_perturbTranslation[0] = 0.0f;
	m_perturbTranslation[1] = 0.0f;
	m_perturbTranslation[2] = 0.0f;
	m_perturbTranslation[3] = 0.0f;

	m_perturbScale[0] = 0.3f;
	m_perturbScale[1] = 0.3f;
	m_perturbScale[2] = 0.0f;
	m_perturbScale[3] = 0.0f;

	result = InitializeSkyPlane(skyPlaneResolution, skyPlaneWidth, skyPlaneTop, skyPlaneBottom, textureRepeat);
	if (!result)
	{
		return false;
	}

	result = LoadTextures(cloudTextureA, cloudTextureB, perturbTexture);
	if (!result)
	{
		return false;
	}

	return true;
}

bool SkyPlaneModel::InitializeSkyPlane(int skyPlaneResolution, float skyPlaneWidth, float skyPlaneTop, float skyPlaneBottom, int textureRepeat)
{
	float quadSize, radius, constant, textureDelta;
	int index, index1, index2, index3, index4;
	float positionX, positionY, positionZ, tu, tv;
	VertexType* skyPlane;

	skyPlane = new VertexType[(skyPlaneResolution + 1) * (skyPlaneResolution + 1)];
	if (!skyPlane)
	{
		return false;
	}

	quadSize = skyPlaneWidth / (float)skyPlaneResolution;

	radius = skyPlaneWidth / 2.0f;

	constant = (skyPlaneTop - skyPlaneBottom) / (radius * radius);

	textureDelta = (float)textureRepeat / (float)skyPlaneResolution;

	for (int j = 0; j <= skyPlaneResolution; j++)
	{
		for (int i = 0; i <= skyPlaneResolution; i++)
		{
			positionX = (-0.5f * skyPlaneWidth) + ((float)i * quadSize);
			positionZ = (-0.5f * skyPlaneWidth) + ((float)j * quadSize);
			positionY = skyPlaneTop - (constant * ((positionX * positionX) + (positionZ * positionZ)));

			tu = (float)i * textureDelta;
			tv = (float)j * textureDelta;

			index = j * (skyPlaneResolution + 1) + i;

			skyPlane[index].x = positionX;
			skyPlane[index].y = positionY;
			skyPlane[index].z = positionZ;
			skyPlane[index].tu = tu;
			skyPlane[index].tv = tv;
		}
	}

	m_vertexCount = ((skyPlaneResolution + 1) * (skyPlaneResolution + 1)) * 6;

	m_vertices = new VertexType[m_vertexCount];
	if (!m_vertices)
	{
		return false;
	}

	index = 0;

	for (int j = 0; j < skyPlaneResolution; j++)
	{
		for (int i = 0; i < skyPlaneResolution; i++)
		{
			index1 = j * (skyPlaneResolution + 1) + i;
			index2 = j * (skyPlaneResolution + 1) + (i + 1);
			index3 = (j + 1) * (skyPlaneResolution + 1) + i;
			index4 = (j + 1) * (skyPlaneResolution + 1) + (i + 1);

			//Triangle 1
			m_vertices[index].x = skyPlane[index1].x;
			m_vertices[index].y = skyPlane[index1].y;
			m_vertices[index].z = skyPlane[index1].z;
			m_vertices[index].tu = skyPlane[index1].tu;
			m_vertices[index].tv = skyPlane[index1].tv;
			index++;

			m_vertices[index].x = skyPlane[index2].x;
			m_vertices[index].y = skyPlane[index2].y;
			m_vertices[index].z = skyPlane[index2].z;
			m_vertices[index].tu = skyPlane[index2].tu;
			m_vertices[index].tv = skyPlane[index2].tv;
			index++;

			m_vertices[index].x = skyPlane[index3].x;
			m_vertices[index].y = skyPlane[index3].y;
			m_vertices[index].z = skyPlane[index3].z;
			m_vertices[index].tu = skyPlane[index3].tu;
			m_vertices[index].tv = skyPlane[index3].tv;
			index++;

			//Triangle 2
			m_vertices[index].x = skyPlane[index3].x;
			m_vertices[index].y = skyPlane[index3].y;
			m_vertices[index].z = skyPlane[index3].z;
			m_vertices[index].tu = skyPlane[index3].tu;
			m_vertices[index].tv = skyPlane[index3].tv;
			index++;

			m_vertices[index].x = skyPlane[index2].x;
			m_vertices[index].y = skyPlane[index2].y;
			m_vertices[index].z = skyPlane[index2].z;
			m_vertices[index].tu = skyPlane[index2].tu;
			m_vertices[index].tv = skyPlane[index2].tv;
			index++;

			m_vertices[index].x = skyPlane[index4].x;
			m_vertices[index].y = skyPlane[index4].y;
			m_vertices[index].z = skyPlane[index4].z;
			m_vertices[index].tu = skyPlane[index4].tu;
			m_vertices[index].tv = skyPlane[index4].tv;
			index++;
		}
	}

	delete[] skyPlane;
	skyPlane = nullptr;

	return true;
}


bool SkyPlaneModel::LoadTextures(const char* cloudTextureA, const char* cloudTextureB, char* perturbTexture)
{
	bool result;

	m_cloudTextureA = SOIL_load_OGL_texture("test001.png", SOIL_LOAD_AUTO, SOIL_CREATE_NEW_ID, SOIL_FLAG_MIPMAPS | SOIL_FLAG_NTSC_SAFE_RGB | SOIL_FLAG_COMPRESS_TO_DXT);

	m_cloudTextureB = SOIL_load_OGL_texture(cloudTextureB, SOIL_LOAD_AUTO, SOIL_CREATE_NEW_ID, SOIL_FLAG_MIPMAPS | SOIL_FLAG_NTSC_SAFE_RGB | SOIL_FLAG_COMPRESS_TO_DXT);

	m_perturbTexture = new BumpMap;
	if (!m_perturbTexture)
	{
		return false;
	}

	result = m_perturbTexture->Initialize(perturbTexture);
	if (!result)
	{
		return false;
	}

	return true;
}

void SkyPlaneModel::Render()
{
	float sampleLocation[2];
	Vector3 perturbValue;

	glBindTexture(GL_TEXTURE_2D, m_cloudTextureA);
	glTexEnvf(GL_TEXTURE_ENV, GL_TEXTURE_ENV_MODE, GL_MODULATE);

	CombineTextures(m_cloudTextureA, m_cloudTextureB, 0.5f);

	glEnable(GL_BLEND);
	glBlendFunc(GL_SRC_ALPHA, GL_ONE);

	glBegin(GL_TRIANGLES);

	for (int i = 0; i < m_vertexCount; i++)
	{
		sampleLocation[0] = m_vertices[i].tu + m_perturbTranslation[0];
		sampleLocation[1] = m_vertices[i].tv;

		/*
		perturbValue = m_perturbTexture->SampleBumpMap(sampleLocation[0], sampleLocation[1]);

		perturbValue.scale(m_perturbScale[0]);
		
		perturbValue.x = perturbValue.x + m_vertices[i].tu + m_perturbTranslation[0];
		perturbValue.y = perturbValue.y + m_vertices[i].tv + m_perturbTranslation[0];
		*/

		glMultiTexCoord2f(GL_TEXTURE0, sampleLocation[0], sampleLocation[1]);
		
		sampleLocation[0] = m_vertices[i].tu + m_perturbTranslation[1];
		sampleLocation[1] = m_vertices[i].tv;

		/*
		perturbValue = m_perturbTexture->SampleBumpMap(sampleLocation[0], sampleLocation[1]);

		perturbValue.scale(m_perturbScale[1]);
				
		perturbValue.x = perturbValue.x + m_vertices[i].tu + m_perturbTranslation[1];
		perturbValue.y = perturbValue.y + m_vertices[i].tv + m_perturbTranslation[1];
		*/

		glMultiTexCoord2f(GL_TEXTURE1, sampleLocation[0], sampleLocation[1]);

		glColor3f(m_brightness, m_brightness, m_brightness);
		glVertex3f(m_vertices[i].x, m_vertices[i].y, m_vertices[i].z);
	}

	glEnd();

	glTexEnvf(GL_TEXTURE_ENV, GL_TEXTURE_ENV_MODE, GL_MODULATE);

	glDisable(GL_BLEND);

	CleanUpTexEnv();
}

void SkyPlaneModel::Frame(float dt)
{
	m_perturbTranslation[0] += 0.0001;
	m_perturbTranslation[1] += 0.0002;

	if (m_perturbTranslation[0] > 1.0){ m_perturbTranslation[0] -= 1.0; }
	if (m_perturbTranslation[1] > 1.0){ m_perturbTranslation[1] -= 1.0; }
}

void SkyPlaneModel::Shutdown()
{
	if (m_perturbTexture)
	{
		m_perturbTexture->Shutdown();
		delete m_perturbTexture;
		m_perturbTexture = nullptr;
	}

	if (m_vertices)
	{
		delete[] m_vertices;
		m_vertices = nullptr;
	}
}