#ifndef _THREADCHANNEL_H_
#define _THREADCHANNEL_H_

#include <mutex>
#include <thread>
#include <list>
#include <sstream>

#include "Semaphore.h"

using std::mutex;
using std::condition_variable;
using std::unique_lock;
using std::list;

typedef unique_lock<mutex> mutex_lock;

template<class type> class channel
{
private:
	list<type> buffer;
	mutex buffer_mutex;
	semaphore sem{ 0 };
	bool channel_poisoned = false;

public:
	void write(type data)
	{
		mutex_lock lock(buffer_mutex);
		buffer.push_back(data);
		sem.signal();
		lock.unlock();
	}

	void read(type& item)
	{
		if (!channel_poisoned)
		{
			sem.wait();
			mutex_lock lock(buffer_mutex);
			if (buffer.empty())
				return;
			item = buffer.front();
			buffer.pop_front();
			lock.unlock();
		}
	}

	bool channel_empty()
	{
		return buffer.empty();
	}

	void release_all()
	{
		channel_poisoned = true;
		sem.release_all();
	}

	void unpoisoned()
	{
		channel_poisoned = false;
	}

};

#endif