#include "Vector3.h"
 
Vector3::Vector3(float x, float y, float z){
    this->x = x;
    this->y = y;
    this->z = z;
}
  
Vector3 Vector3::copy(){
        Vector3 copy(
                this->x,
                this->y,     
                this->z);
        return copy;           
}
 
bool Vector3::equals(const Vector3& v2, float epsilon){
        return ((abs(this->x - v2.x) < epsilon) &&
                    (abs(this->y - v2.y) < epsilon) &&
                        (abs(this->z - v2.z) < epsilon));      
}

bool Vector3::equals(const Vector3& v2)
{
	return equals(v2, 0.00001f);
}
 
 
float Vector3::length(){
        return sqrt(this->lengthSquared());
}
 
float Vector3::lengthSquared(){
        return (
                this->x*this->x +
                this->y*this->y +
                this->z*this->z
                );
}
 
void Vector3::normalise(){
    float mag = this->length();
    if(mag){
            float multiplier = 1.0f/mag;
            this->x *= multiplier;
            this->y *= multiplier;
            this->z *= multiplier;
    }
}
 
Vector3 Vector3::cross(const Vector3& v2){
        Vector3 cross(
                (this->y * v2.z - this->z * v2.y),
                (this->z * v2.x - this->x * v2.z),
                (this->x * v2.y - this->y * v2.x)
        );
        return cross;
}
 
void Vector3::subtract(const Vector3& v1, float scale){
	this->x -= (v1.x*scale);
    this->y -= (v1.y*scale);
    this->z -= (v1.z*scale);         
}
 
void Vector3::set(float x, float y, float z){
        this->x = x;
        this->y = y;
        this->z = z;
}
 
void Vector3::setX(float x){
        this->x = x;
}
 
void Vector3::setY(float y){
        this->y = y;
}
 
void Vector3::setZ(float z){
        this->z = z;
}

float Vector3::getX(){
       return this->x;
}
 
float Vector3::getY(){
       return this->y;
}
 
float Vector3::getZ(){
      return this->z;
}
 
float Vector3::dot(const Vector3& v2){
        return (this->x*v2.x +
                this->y*v2.y +
                this->z*v2.z
                );
}
 
void Vector3::scale(float scale){
	this->x*scale;
    this->y*scale;
    this->z*scale;
}
 
void Vector3::add(const Vector3& v1, float scale){
	this->x += (v1.x*scale);
    this->y += (v1.y*scale);
    this->z += (v1.z*scale);
}
 
Vector3 Vector3::operator+(const Vector3& v2) {
	return Vector3(this->x + v2.x, this->y+v2.y, this->z+v2.z);
}
 
Vector3 Vector3::operator-(const Vector3& v2) {
    return Vector3(this->x - v2.x, this->y - v2.y, this->z - v2.z);
}
 
Vector3& Vector3::operator+=(const Vector3& v2) {
	this->x += v2.x;
	this->y += v2.y;
	this->z += v2.z;
	return *this;
}
 
Vector3& Vector3::operator-=(const Vector3& v2) {
	this->x -= v2.x;
	this->y -= v2.y;
	this->z -= v2.z;
	return *this;
}